package com.sun.org.apache.xerces.internal.xinclude;

import com.sun.org.apache.xerces.internal.util.XMLChar;
import com.sun.org.apache.xerces.internal.impl.io.Latin1Reader;
import com.sun.org.apache.xerces.internal.impl.io.ASCIIReader;
import com.sun.org.apache.xerces.internal.impl.io.UTF16Reader;
import com.sun.org.apache.xerces.internal.impl.io.UTF8Reader;
import com.sun.org.apache.xerces.internal.util.MessageFormatter;
import java.util.Iterator;
import java.net.URLConnection;
import java.io.InputStream;
import java.io.InputStreamReader;
import com.sun.org.apache.xerces.internal.util.EncodingMap;
import java.util.Locale;
import java.io.BufferedInputStream;
import java.util.Map;
import com.sun.org.apache.xerces.internal.util.HTTPInputSource;
import java.net.HttpURLConnection;
import java.net.URL;
import com.sun.org.apache.xerces.internal.impl.XMLEntityManager;
import java.io.IOException;
import com.sun.org.apache.xerces.internal.xni.XMLString;
import com.sun.org.apache.xerces.internal.impl.XMLErrorReporter;
import com.sun.org.apache.xerces.internal.xni.parser.XMLInputSource;
import java.io.Reader;

public class XIncludeTextReader
{
    private Reader fReader;
    private final XIncludeHandler fHandler;
    private XMLInputSource fSource;
    private XMLErrorReporter fErrorReporter;
    private XMLString fTempString;
    
    public XIncludeTextReader(XMLInputSource source, XIncludeHandler handler, int bufferSize) throws IOException {
        this.fTempString = new XMLString();
        this.fHandler = handler;
        this.fSource = source;
        this.fTempString = new XMLString(new char[bufferSize + 1], 0, 0);
    }
    
    public void setErrorReporter(XMLErrorReporter errorReporter) {
        this.fErrorReporter = errorReporter;
    }
    
    protected Reader getReader(XMLInputSource source) throws IOException {
        if (source.getCharacterStream() != null) {
            return source.getCharacterStream();
        }
        InputStream stream = null;
        String encoding = source.getEncoding();
        if (encoding == null) {
            encoding = "UTF-8";
        }
        if (source.getByteStream() != null) {
            stream = source.getByteStream();
            if (!(stream instanceof BufferedInputStream)) {
                stream = new BufferedInputStream(stream, this.fTempString.ch.length);
            }
        }
        else {
            String expandedSystemId = XMLEntityManager.expandSystemId(source.getSystemId(), source.getBaseSystemId(), false);
            URL url = new URL(expandedSystemId);
            URLConnection urlCon = url.openConnection();
            if (urlCon instanceof HttpURLConnection && source instanceof HTTPInputSource) {
                HttpURLConnection urlConnection = (HttpURLConnection)urlCon;
                HTTPInputSource httpInputSource = (HTTPInputSource)source;
                Iterator<Map.Entry<String, String>> propIter = httpInputSource.getHTTPRequestProperties();
                while (propIter.hasNext()) {
                    Map.Entry<Object, Object> entry = (Map.Entry)propIter.next();
                    urlConnection.setRequestProperty((String)entry.getKey(), (String)entry.getValue());
                }
                boolean followRedirects = httpInputSource.getFollowHTTPRedirects();
                if (!followRedirects) {
                    XMLEntityManager.setInstanceFollowRedirects(urlConnection, followRedirects);
                }
            }
            stream = new BufferedInputStream(urlCon.getInputStream());
            String rawContentType = urlCon.getContentType();
            int index = (rawContentType != null) ? rawContentType.indexOf(59) : -1;
            String charset = null;
            String contentType;
            if (index != -1) {
                contentType = rawContentType.substring(0, index).trim();
                charset = rawContentType.substring(index + 1).trim();
                if (charset.startsWith("charset=")) {
                    charset = charset.substring(8).trim();
                    if ((charset.charAt(0) == '\"' && charset.charAt(charset.length() - 1) == '\"') || (charset.charAt(0) == '\'' && charset.charAt(charset.length() - 1) == '\'')) {
                        charset = charset.substring(1, charset.length() - 1);
                    }
                }
                else {
                    charset = null;
                }
            }
            else {
                contentType = ((rawContentType != null) ? rawContentType.trim() : "");
            }
            String detectedEncoding = null;
            if (contentType.equals("text/xml")) {
                if (charset != null) {
                    detectedEncoding = charset;
                }
                else {
                    detectedEncoding = "US-ASCII";
                }
            }
            else if (contentType.equals("application/xml")) {
                if (charset != null) {
                    detectedEncoding = charset;
                }
                else {
                    detectedEncoding = this.getEncodingName(stream);
                }
            }
            else if (contentType.endsWith("+xml")) {
                detectedEncoding = this.getEncodingName(stream);
            }
            if (detectedEncoding != null) {
                encoding = detectedEncoding;
            }
        }
        encoding = encoding.toUpperCase(Locale.ENGLISH);
        encoding = this.consumeBOM(stream, encoding);
        if (encoding.equals("UTF-8")) {
            return this.createUTF8Reader(stream);
        }
        if (encoding.equals("UTF-16BE")) {
            return this.createUTF16Reader(stream, true);
        }
        if (encoding.equals("UTF-16LE")) {
            return this.createUTF16Reader(stream, false);
        }
        String javaEncoding = EncodingMap.getIANA2JavaMapping(encoding);
        if (javaEncoding == null) {
            MessageFormatter aFormatter = this.fErrorReporter.getMessageFormatter("http://www.w3.org/TR/1998/REC-xml-19980210");
            Locale aLocale = this.fErrorReporter.getLocale();
            throw new IOException(aFormatter.formatMessage(aLocale, "EncodingDeclInvalid", new Object[] { encoding }));
        }
        if (javaEncoding.equals("ASCII")) {
            return this.createASCIIReader(stream);
        }
        if (javaEncoding.equals("ISO8859_1")) {
            return this.createLatin1Reader(stream);
        }
        return new InputStreamReader(stream, javaEncoding);
    }
    
    private Reader createUTF8Reader(InputStream stream) {
        return new UTF8Reader(stream, this.fTempString.ch.length, this.fErrorReporter.getMessageFormatter("http://www.w3.org/TR/1998/REC-xml-19980210"), this.fErrorReporter.getLocale());
    }
    
    private Reader createUTF16Reader(InputStream stream, boolean isBigEndian) {
        return new UTF16Reader(stream, this.fTempString.ch.length << 1, isBigEndian, this.fErrorReporter.getMessageFormatter("http://www.w3.org/TR/1998/REC-xml-19980210"), this.fErrorReporter.getLocale());
    }
    
    private Reader createASCIIReader(InputStream stream) {
        return new ASCIIReader(stream, this.fTempString.ch.length, this.fErrorReporter.getMessageFormatter("http://www.w3.org/TR/1998/REC-xml-19980210"), this.fErrorReporter.getLocale());
    }
    
    private Reader createLatin1Reader(InputStream stream) {
        return new Latin1Reader(stream, this.fTempString.ch.length);
    }
    
    protected String getEncodingName(InputStream stream) throws IOException {
        byte[] b4 = new byte[4];
        String encoding = null;
        stream.mark(4);
        int count = stream.read(b4, 0, 4);
        stream.reset();
        if (count == 4) {
            encoding = this.getEncodingName(b4);
            return encoding;
        }
        return encoding;
    }
    
    protected String consumeBOM(InputStream stream, String encoding) throws IOException {
        byte[] b = new byte[3];
        int count = 0;
        stream.mark(3);
        if (encoding.equals("UTF-8")) {
            count = stream.read(b, 0, 3);
            if (count == 3) {
                int b2 = b[0] & 0xFF;
                int b3 = b[1] & 0xFF;
                int b4 = b[2] & 0xFF;
                if (b2 != 239 || b3 != 187 || b4 != 191) {
                    stream.reset();
                }
            }
            else {
                stream.reset();
            }
        }
        else if (encoding.startsWith("UTF-16")) {
            count = stream.read(b, 0, 2);
            if (count == 2) {
                int b2 = b[0] & 0xFF;
                int b3 = b[1] & 0xFF;
                if (b2 == 254 && b3 == 255) {
                    return "UTF-16BE";
                }
                if (b2 == 255 && b3 == 254) {
                    return "UTF-16LE";
                }
            }
            stream.reset();
        }
        return encoding;
    }
    
    protected String getEncodingName(byte[] b4) {
        int b5 = b4[0] & 0xFF;
        int b6 = b4[1] & 0xFF;
        if (b5 == 254 && b6 == 255) {
            return "UTF-16BE";
        }
        if (b5 == 255 && b6 == 254) {
            return "UTF-16LE";
        }
        int b7 = b4[2] & 0xFF;
        if (b5 == 239 && b6 == 187 && b7 == 191) {
            return "UTF-8";
        }
        int b8 = b4[3] & 0xFF;
        if (b5 == 0 && b6 == 0 && b7 == 0 && b8 == 60) {
            return "ISO-10646-UCS-4";
        }
        if (b5 == 60 && b6 == 0 && b7 == 0 && b8 == 0) {
            return "ISO-10646-UCS-4";
        }
        if (b5 == 0 && b6 == 0 && b7 == 60 && b8 == 0) {
            return "ISO-10646-UCS-4";
        }
        if (b5 == 0 && b6 == 60 && b7 == 0 && b8 == 0) {
            return "ISO-10646-UCS-4";
        }
        if (b5 == 0 && b6 == 60 && b7 == 0 && b8 == 63) {
            return "UTF-16BE";
        }
        if (b5 == 60 && b6 == 0 && b7 == 63 && b8 == 0) {
            return "UTF-16LE";
        }
        if (b5 == 76 && b6 == 111 && b7 == 167 && b8 == 148) {
            return "CP037";
        }
        return null;
    }
    
    public void parse() throws IOException {
        this.fReader = this.getReader(this.fSource);
        this.fSource = null;
        int readSize = this.fReader.read(this.fTempString.ch, 0, this.fTempString.ch.length - 1);
        this.fHandler.fHasIncludeReportedContent = true;
        while (readSize != -1) {
            for (int i = 0; i < readSize; ++i) {
                char ch = this.fTempString.ch[i];
                if (!this.isValid((int)ch)) {
                    if (XMLChar.isHighSurrogate((int)ch)) {
                        int ch2;
                        if (++i < readSize) {
                            ch2 = this.fTempString.ch[i];
                        }
                        else {
                            ch2 = this.fReader.read();
                            if (ch2 != -1) {
                                this.fTempString.ch[readSize++] = (char)ch2;
                            }
                        }
                        if (XMLChar.isLowSurrogate(ch2)) {
                            int sup = XMLChar.supplemental(ch, (char)ch2);
                            if (!this.isValid(sup)) {
                                this.fErrorReporter.reportError("http://www.w3.org/TR/1998/REC-xml-19980210", "InvalidCharInContent", new Object[] { Integer.toString(sup, 16) }, (short)2);
                            }
                        }
                        else {
                            this.fErrorReporter.reportError("http://www.w3.org/TR/1998/REC-xml-19980210", "InvalidCharInContent", new Object[] { Integer.toString(ch2, 16) }, (short)2);
                        }
                    }
                    else {
                        this.fErrorReporter.reportError("http://www.w3.org/TR/1998/REC-xml-19980210", "InvalidCharInContent", new Object[] { Integer.toString((int)ch, 16) }, (short)2);
                    }
                }
            }
            if (this.fHandler != null && readSize > 0) {
                this.fTempString.offset = 0;
                this.fTempString.length = readSize;
                this.fHandler.characters(this.fTempString, this.fHandler.modifyAugmentations(null, true));
            }
            readSize = this.fReader.read(this.fTempString.ch, 0, this.fTempString.ch.length - 1);
        }
    }
    
    public void setInputSource(XMLInputSource source) {
        this.fSource = source;
    }
    
    public void close() throws IOException {
        if (this.fReader != null) {
            this.fReader.close();
            this.fReader = null;
        }
    }
    
    protected boolean isValid(int ch) {
        return XMLChar.isValid(ch);
    }
    
    protected void setBufferSize(int bufferSize) {
        if (this.fTempString.ch.length != ++bufferSize) {
            this.fTempString.ch = new char[bufferSize];
        }
    }
}
